/////////////////////////////////////////////////////////////////////////
////                                                                 ////
////                        usb_kbmouse_jfmc.c                       ////
//// Sends key stroke values on various input changes                ////
////                                                                 ////
//// For a more generic USB HID example (as well as some HID         ////
//// documentation) see ex_usb_hid.c                                 ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////                                                                 ////
//// VERSION HISTORY                                                 ////
////                                                                 ////
//// 03/03/2013 : Initial 18fxx5x release.                           ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////

//set to 1 to use a PIC's internal USB Peripheral
//set to 0 to use a National USBN960x peripheral
#define __USB_PIC_PERIF__ 1
#include <18F2550.h>
#fuses HSPLL,NOWDT,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL5,CPUDIV1,VREGEN
#use delay(clock=48000000)

/////////////////////////////////////////////////////////////////////////////
//
// CCS Library dynamic defines.  For dynamic configuration of the CCS Library
// for your application several defines need to be made.  See the comments
// at usb.h for more information
//
/////////////////////////////////////////////////////////////////////////////

//Tells the CCS PIC USB firmware to include HID handling code.
#DEFINE USB_HID_DEVICE  TRUE

//the following defines needed for the CCS USB PIC driver to enable the TX endpoint 1
#define USB_EP1_TX_ENABLE  USB_ENABLE_INTERRUPT   //turn on EP1 for IN bulk/interrupt transfers
#define USB_EP1_TX_SIZE 8

#define USB_EP1_RX_ENABLE  USB_ENABLE_INTERRUPT   //turn on EP1 for IN bulk/interrupt transfers
#define USB_EP1_RX_SIZE 8

/////////////////////////////////////////////////////////////////////////////
//
// Include the CCS USB Libraries.  See the comments at the top of these
// files for more information
//
/////////////////////////////////////////////////////////////////////////////
#include <pic18_usb.h>
#include <usb_desc_kbmouse2.h>    //USB Configuration and Device descriptors for this UBS device
#include <usb.c>        //handles usb setup tokens and get descriptor reports


/////////////////////////////////////////////////////////////////////////////
//
// Configure the demonstration I/O
//
/////////////////////////////////////////////////////////////////////////////

#define BUTTON1     PIN_C0
#define BUTTON2     PIN_C1
#define BUTTON3     PIN_C2
#define BUTTON4     PIN_C6
#define BUTTON5     PIN_C7
#define BUTTON6     PIN_B0
#define BUTTON7     PIN_B1
#define BUTTON8     PIN_B2
#define BUTTON9     PIN_B3
#define BUTTON10    PIN_B4
#define BUTTON11    PIN_B5
#define BUTTON12    PIN_A1
#define BUTTON13    PIN_A2
#define BUTTON14    PIN_A3
#define BUTTON15    PIN_A4
#define BUTTON16    PIN_A5

#define HID_KEYPRESS       !input
#define HID_KEY_DELAY()    delay_ms( 10 )
#define HID_KEY_DEBOUNCE() delay_ms( 75 )

#define MAX_CNT_BUTTONS 16

#DEFINE LED1         PIN_A0
#define LED1_OFF()   output_high( LED1 )
#define LED1_ON()    output_low( LED1 )

static unsigned int8 arrBUTTON_VALUES[ MAX_CNT_BUTTONS + 2 ] = { '0','1','2','3','4','5','6','7','8','9','a','b','c','d',0x2A/*<VK_BACKSPACE>*/, 0x28/*<VK_ENTER>*/ , 0x00, } ;
static unsigned int8 arrBUTTON_USB_SCANCODES[ MAX_CNT_BUTTONS + 2 ] = { 0x00, } ;
// 'a' = 4 , 'b' = 5 , etc...  
// thus, 'a' = ( ( 'a' - 'a' ) + 4 )

// '1' = 1E , '2' = 1F , etc...  
// thus, '1' = ( ( '1' - '1' ) + 0x1E )

// value of 0x00 indicates that the input is disabled.

static unsigned int8 arrBUTTON_STATES[ MAX_CNT_BUTTONS + 2 ]    = { 0x00 , } ;
static unsigned int8 arrBUTTON_PREVSTATES[ MAX_CNT_BUTTONS + 2 ] = { 0x00 , } ;

#define MAINLOOP_WAIT() delay_ms( 10 )

#define EEPROM_LOC_WATERMARK  1
#define EEPROM_LOC_BUTTONS    2
#define EEPROM_VAL_WATERMARK  'X'


/////////////////////////////////////////////////////////////////////////////
//
// usb_keyboard_task()
//
// Sends a packet of keyboard data.  The protocol was specified in the HID
// report descriptor (see usb_desc_kbmouse.h), and is:
//     tx_msg[0] = HID report id (2)
//     tx_msg[1] = modifier (an 8bit bitmap of shift, tab, alt keypress)
//     tx_msg[2] = const 0
//     tx_msg[3:7] = an array of held down keys.  a=4, b=5, etc.
//                   if msg[2:7]={0} then no keys are held down
//
//     rx_msg[1] = HID report id (2)
//     rx_msg[0] = 5bit bitmap of led status
//
//===========================================================================
void usb_keyboard_task(void) {
  int8 i = 0 ; 
  boolean bSentKey = FALSE ;
  static int8 tx_msg[8]={2,0,0,0,0,0,0,0} ;

  //read all 12 buttons
  //we could read port values instead of individual reads. no rush...
  arrBUTTON_STATES[ 0 ]  = HID_KEYPRESS( BUTTON1 ) ;
  arrBUTTON_STATES[ 1 ]  = HID_KEYPRESS( BUTTON2 ) ;
  arrBUTTON_STATES[ 2 ]  = HID_KEYPRESS( BUTTON3 ) ;
  arrBUTTON_STATES[ 3 ]  = HID_KEYPRESS( BUTTON4 ) ;
  arrBUTTON_STATES[ 4 ]  = HID_KEYPRESS( BUTTON5 ) ;
  arrBUTTON_STATES[ 5 ]  = HID_KEYPRESS( BUTTON6 ) ;
  arrBUTTON_STATES[ 6 ]  = HID_KEYPRESS( BUTTON7 ) ;
  arrBUTTON_STATES[ 7 ]  = HID_KEYPRESS( BUTTON8 ) ;
  arrBUTTON_STATES[ 8 ]  = HID_KEYPRESS( BUTTON9 ) ;
  arrBUTTON_STATES[ 9 ]  = HID_KEYPRESS( BUTTON10 ) ;
  arrBUTTON_STATES[ 10 ] = HID_KEYPRESS( BUTTON11 ) ;
  arrBUTTON_STATES[ 11 ] = HID_KEYPRESS( BUTTON12 ) ;
  arrBUTTON_STATES[ 12 ] = HID_KEYPRESS( BUTTON13 ) ;
  arrBUTTON_STATES[ 13 ] = HID_KEYPRESS( BUTTON14 ) ;
  arrBUTTON_STATES[ 14 ] = HID_KEYPRESS( BUTTON15 ) ;
  arrBUTTON_STATES[ 15 ] = HID_KEYPRESS( BUTTON16 ) ;

  //send the values
  for( i = 0 ; i < MAX_CNT_BUTTONS ; i++ ) {

    if ( arrBUTTON_USB_SCANCODES[ i ] == 0x00 ) //skip an input because it is disabled
      continue ;

    if ( arrBUTTON_STATES[ i ] ) {       
      tx_msg[ 3 ] = arrBUTTON_USB_SCANCODES[ i ] ;
      LED1_ON() ;
    }//end ( arrBUTTON_STATES[ i ] ) if
    else {
      tx_msg[ 3 ] = 0x00 ;
    }//end else

    if ( arrBUTTON_STATES[ i ] != arrBUTTON_PREVSTATES[ i ] ) {
      //usb_put_packet( 1 , tx_msg , sizeof( tx_msg ) , USB_DTS_TOGGLE ) ;
      usb_put_packet( 1 , tx_msg , 8 , USB_DTS_TOGGLE ) ;
      HID_KEY_DELAY() ;
      usb_task() ;
      bSentKey = TRUE ;
    }//end ( arrBUTTON_STATES[ i ] != arrBUTTON_PREVSTATES[ i ] ) if 

  }//end for

  memcpy( arrBUTTON_PREVSTATES , arrBUTTON_STATES , sizeof( arrBUTTON_STATES  ) ) ;

  //debounce keys
  if ( bSentKey )
    HID_KEY_DEBOUNCE() ;

}//end usb_keyboard_task() func
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
//
// usb_rx_task()
//
// Listens to EP1 for any incoming packets.  The only report ID that is
// configurd to send us data is 2 (keyboard LED status, see above)
//
//===========================================================================
void usb_rx_task(void) {
   int8 rx_msg[USB_EP1_RX_SIZE];

   if ( usb_kbhit( 1 ) ) {
      usb_get_packet( 1 , rx_msg , sizeof( rx_msg ) ) ;
   }//end if

}//end ISR
//===========================================================================

//===========================================================================
//
//===========================================================================
void Fill_USB_SCANCODES( void ) {
  int8 i = 0 ; 

  for( i = 0 ; i < MAX_CNT_BUTTONS ; i++ ) {
         if ( ( arrBUTTON_VALUES[ i ] >= 'a' ) && ( arrBUTTON_VALUES[ i ] <= 'z' ) )
      arrBUTTON_USB_SCANCODES[ i ] = ( ( arrBUTTON_VALUES[ i ] - 'a' ) + 4 ) ;
    else if ( ( arrBUTTON_VALUES[ i ] >= 'A' ) && ( arrBUTTON_VALUES[ i ] <= 'Z' ) )
      arrBUTTON_USB_SCANCODES[ i ] = ( ( arrBUTTON_VALUES[ i ] - 'A' ) + 4 ) ;
    else if ( ( arrBUTTON_VALUES[ i ] >= '1' ) && ( arrBUTTON_VALUES[ i ] <= '9' ) )
      arrBUTTON_USB_SCANCODES[ i ] = ( ( arrBUTTON_VALUES[ i ] - '1' ) + 0x1E ) ;
    else if ( arrBUTTON_VALUES[ i ] == '0' )
      arrBUTTON_USB_SCANCODES[ i ] = 0x27 ;
    else if ( arrBUTTON_VALUES[ i ] == 0x28 ) //return
      arrBUTTON_USB_SCANCODES[ i ] = 0x28 ;
    else if ( arrBUTTON_VALUES[ i ] == 0x2A ) //backspace
      arrBUTTON_USB_SCANCODES[ i ] = 0x2A ;
    else if ( arrBUTTON_VALUES[ i ] == 0x2B ) //tab
      arrBUTTON_USB_SCANCODES[ i ] = 0x2B ;
    else
      arrBUTTON_USB_SCANCODES[ i ] = 0x00 ;
  }//end for


}//end func
//===========================================================================

//===========================================================================
//
//===========================================================================
void LoadEEPROMcfg( void ) {
  int8 j = 0 ;
  
  if ( EEPROM_VAL_WATERMARK == read_eeprom( EEPROM_LOC_WATERMARK ) ) {
    
    memset( arrBUTTON_VALUES , sizeof( arrBUTTON_VALUES ) , 0x00 ) ;
    
    for ( j = 0 ; j < MAX_CNT_BUTTONS ; j++ ) {
      arrBUTTON_VALUES[ j ] = read_eeprom( EEPROM_LOC_BUTTONS + j ) ;
      //delay_ms( 2 ) ;
    }//end for

  }//end if
  else {

    for ( j = 0 ; j < MAX_CNT_BUTTONS ; j++ ) {
      write_eeprom( EEPROM_LOC_BUTTONS + j , arrBUTTON_VALUES[ j ] ) ;
      delay_ms( 20 ) ;
    }//end for

    write_eeprom( EEPROM_LOC_WATERMARK , EEPROM_VAL_WATERMARK ) ;
    delay_ms( 20 ) ;

  }//end else

}//end LoadEEPROMcfg( void ) func
//===========================================================================


//===========================================================================
// main
//===========================================================================
void main() {
  int8 scntr ;
  
  setup_adc_ports ( NO_ANALOGS ) ;
  setup_adc( ADC_OFF ) ;
  
  set_tris_a( 0b11111110 ) ;
  #use fast_io( A )
  set_tris_b( 0b11111111 ) ;
  #use fast_io( B )
  set_tris_c( 0b11111111 ) ;

  LED1_ON() ;

  SETUP_WDT( WDT_OFF ) ;
  SETUP_TIMER_0( T0_OFF ) ;
  SETUP_TIMER_1( T1_DISABLED ) ;
  SETUP_TIMER_2( T2_DISABLED , 255 , 16 ) ;
  SETUP_TIMER_3( T3_DISABLED ) ;
  
  
  usb_init() ;

  LoadEEPROMcfg() ;      // Must be called first
  Fill_USB_SCANCODES() ;   // Must be called second

  for( scntr = 0 ; scntr < 33 ; scntr++ ) { // startup loop
    delay_ms( 33 ) ;
    usb_task() ;
  }//end startup for loop

  while ( TRUE ) { // main infinite while loop

    MAINLOOP_WAIT() ;
    usb_task() ;
    LED1_OFF() ;

    if ( usb_enumerated() ) {
      usb_keyboard_task() ;
      usb_rx_task() ;
    }//end usb_enumerated() if

  }//end main infinite while loop

}//end main
//===========================================================================


